###################################################################################################
## this will create an instance based on ami of last saved. or you can supply one from command line
## it assumes you save the ami in "ami.txt" automatically by running "done.ps1"
###################################################################################################

$mypem="d:\amazon\myAmiTestKey.pem"
$mykey="myAmiTestKey"
$mytype="m1.small"
$myzone="us-east-1d"


#######################################################################
## The ami is by command line argument or from ami.txt
#######################################################################

	if ( $args.length -eq 1 )
	{
		$myami=$args[0];
	}
	else
	{
		if (test-path -path ami.txt)
		{
			$myami=(cat ami.txt);
		}
		else
		{
			echo "The previous stored ami is not stored in ami.txt, please input new ami."
			return;
		}
	}

#######################################################################
## first let's collect information about image

	echo "let us verify ami $myami first...";
	$result=(ec2-describe-images $myami);
	
	foreach ($line in $result)
	{
		$words=$line -split "`t";
		if ($words[0] -ieq "IMAGE")
		{				
			$availability=$words[4];
			$productioncode=$words[7];
			$platform=$words[11];
		}
	}
	if ($availability -ne "available")
	{
		echo "ami $myami is $availability";
		return ;
	}
	
	if ($productioncode -ne "i386")
	{
		$mytype="m1.large";
		echo "ami is not i386 create instance of $mytype";
	}

	echo "about to create instance with image $myami of platform $platform "	


#######################################################################
## first we calculate spot price. 
#######################################################################

#######################################################################
## define your own instance type here.
$instancetype="m1.small";
#######################################################################

	echo "about to calculate average price...";
	$sum=0;
	$counter=0;
	$result=(ec2-describe-spot-price-history --instance-type $mytype);
	foreach ($line in $result)
	{
		$words=-split $line;
		if ($words[4] -ieq $platform)
		{
			$sum += $words[1];
			$counter+=1;
		}
	}
	if ($counter -ne 0)
	{
		$average = $sum/$counter;
	}
	else
	{
		$average = 0;
	}
	echo "total $sum, counter $counter, average $average";

	
#######################################################################
## define your own bidding rate which will be multiplied by average price
$bidrate=1.2;	
#######################################################################

if ($average -eq 0)
{
	# amazon doesn't allow 0 price for bidding.
	$average=0.08;
}
$bidprice=($average*$bidrate);
	echo "Average price is $average and your bid price will be $bidprice";

#######################################################################3
## First create spot instance request and wait for it to be active

	echo "about to create spot instance with image $myami "	

	$result=-split(ec2-request-spot-instances  $myami -k $mykey -z $myzone --price $bidprice --instance-type $mytype);
	$spotrequestid=$result[1];

	echo "spotrequestid=$spotrequestid"

	do
	{
		$result=(ec2-describe-spot-instance-requests $spotrequestid) -split "`t";
		$state=$result[5];
		write-host "." -nonewline;
		start-sleep 1;
	}
	while ($state -ieq "open");
	write-host "`n";

	if ($state -ieq "active")
	{
		#######################################
		## our spot instance request is successful and need to wait for it be running
		$instanceid=$result[11];

		echo "instance $instanceid created and now wait for it being ready..."
		do
		{
			$status="pending";
			$result=(ec2-describe-instances $instanceid);
			foreach ($line in $result)
			{
				$words=$line -split "`t";
				if ($words[0] -ieq "INSTANCE")
				{				
					$status=$words[5];
					$hostname=$words[3];				
				
				}
			}
				
			write-host "." -nonewline
			start-sleep 1;
		}
		while ($status -ieq "pending");

		write-host "`n";
		if ($status -ieq "running")
		{
			echo "host=$hostname";
			echo "the instance status is $status"
		}
		else
		{
			echo "the instance status is $status"
		}
		
		### save instanceid to file for later using
	
		echo "$instanceid" | out-file -filepath "id.txt"

#		$password=(ec2-get-password $instanceid -k $mypem);
		
#		echo "password for administrator is $password";

		#####################################################
		echo "now we are about to bind the ip with our instance";
		if (test-path -path ip.txt)
		{
			$myip=(cat ip.txt);
		}
		if ($myip.length -lt 4) ##  just be safe for some empty space
		{
			## this means we need to allocate elastic ip for the first time.
			$result=-split (ec2-allocate-address);
			$myip=$result[1];
		}
		echo "the elastic ip is $myip and we are going to bind";
		ec2-associate-address $myip -i $instanceid
		out-file -filepath ip.txt -inputobject $myip;

	}
	else
	{
		echo "spot instance request status is $state";
	}

	









	
	
